from PIL import Image, ImageDraw, ImageFont
from colorama import Style
from ascii_forge.config import ASCII_CHARS, COLOR_MAP


def apply_monochrome_color(ascii_art: str, color: str) -> str:
    """Apply a single color to ASCII Art"""
    return COLOR_MAP.get(color) + ascii_art + Style.RESET_ALL


def image_to_ascii(
    image_path: str,
    width: int = 100,
    invert: bool = False,
    color: str = None,
    save_as_txt: str = "",
    save_as_png: str = "",
) -> str:
    """Main functon to convert image to ASCII"""
    
    if color == "multi" and not save_as_png:
        raise Exception("Use 'multi' option only with '--save-as-png'/'-sp' option")

    image = Image.open(image_path)
    w_orig, h_orig = image.width, image.height

    ASCII_CHARS_USED = ASCII_CHARS[::-1] if invert else ASCII_CHARS
    SCALE = width / w_orig
    INTERVAL = len(ASCII_CHARS) / 256
    CHAR_W, CHAR_H = 4, 8
    FONT_FACTOR = CHAR_W / CHAR_H

    image = image.resize((int(SCALE * w_orig), int(SCALE * h_orig * FONT_FACTOR)))
    w_new, h_new = image.width, image.height

    # logic for text output
    if color != "multi":
        image = image.convert("L")
        pixels = list(image.getdata())

        ascii_str = "".join(ASCII_CHARS_USED[int(pixel * INTERVAL)] for pixel in pixels)
        ascii_art = "\n".join(ascii_str[i : i + w_new] for i in range(0, len(ascii_str), w_new))

        if save_as_txt:
            with open(save_as_txt, "w") as ascii_file:
                ascii_file.write(ascii_art)
            ascii_art = f"Saved ASCII art to {save_as_txt}"
        elif color:
            ascii_art = apply_monochrome_color(ascii_art, color)

        return ascii_art

    # logic for png output
    font = ImageFont.load_default()
    output_img = Image.new("RGBA", (CHAR_W * w_new, CHAR_H * h_new), color=(0, 0, 0, 255))
    draw = ImageDraw.Draw(output_img)

    image = image.convert("RGBA")
    pixels = list(image.getdata())

    for i in range(h_new):
        for j, (r, g, b, a) in enumerate(pixels[i * w_new : (i + 1) * w_new]):
            fill = (r, g, b, a) if color == "multi" else None
            gray = int(r / 3 + g / 3 + b / 3)
            draw.text(
                (j * CHAR_W, i * CHAR_H), font=font, text=ASCII_CHARS_USED[int(gray * INTERVAL)], fill=fill
            )

    output_img.save(save_as_png)
    return f"Saved ASCII art to {save_as_png}"



import click
from ascii_forge.converter import image_to_ascii
from ascii_forge.config import COLOR_MAP


@click.command()
@click.argument("image_path", type=click.Path(exists=True))
@click.option("--width", "-w", default=100, type=int, help="Width of ASCII ouput (may improve clarity)")
@click.option("--invert", "-i", is_flag=True, help="Invert ASCII output")
@click.option(
    "--color",
    "-c",
    default=None,
    type=click.Choice(COLOR_MAP.keys()),
    help="Color of ASCII output. 'multi' value only works with '--save-as-png' option",
)
@click.option("--save-as-txt", "-st", type=click.Path(), help="Save ASCII output to a .txt file")
@click.option("--save-as-png", "-sp", type=click.Path(), help="Save ASCII output to a .png file")
def cli(image_path, width, invert, color, save_as_txt, save_as_png):
    """Convert an IMAGE to ASCII Art"""
    try:
        ascii_art = image_to_ascii(image_path, width, invert, color, save_as_txt, save_as_png)
        click.echo(ascii_art)
    except Exception as e:
        print(str(e))


from setuptools import setup, find_packages

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="ascii-forge",
    version="0.4",
    author="Kanak Tanwar",
    author_email="kanaktanwarpro@gmail.com",
    description="Turn your images into ASCII art from your terminal",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/kanakOS01/ascii-forge",
    project_urls={
        "GitHub Repository": "https://github.com/kanakOS01/ascii-forge",
        "PyPI Project Page": "https://pypi.org/project/ascii-forge/"
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires='>=3.6',
    packages=find_packages(),
    install_requires=[
        "Click", 
        "Pillow", 
        "Colorama"
    ],
    keywords="ascii-forge ascii forge ascii-art art",
    entry_points={
        "console_scripts": [
            "ascii-forge=ascii_forge.__main__:cli",
        ],
    },
)



from colorama import Fore, init

init(autoreset=True)

ASCII_CHARS = " .,:-+=*%#@"

COLOR_MAP = {
    "red": Fore.RED,
    "green": Fore.GREEN,
    "yellow": Fore.YELLOW,
    "blue": Fore.BLUE,
    "magenta": Fore.MAGENTA,
    "cyan": Fore.CYAN,
    "white": Fore.WHITE,
    "multi": "",
}
